package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.data_provider.PlanFeaturesVariant
import com.biganto.visual.roompark.data.data_provider.featuresVariants
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.FlatPlanContract
import io.reactivex.Observable
import io.reactivex.rxkotlin.subscribeBy
import io.reactivex.schedulers.Schedulers
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 20.04.2020.
 */

class PlanTypesUseCase @Inject constructor(
    private val planContract: FlatPlanContract,
    private val auth:AuthContract
) {

    private val fetchAllPlanTypes =
        auth.currentUser()
            .map { user ->
                val list =
                    user.deals?.map { it.estate as EstateEntity }?.toMutableList()
                        ?: mutableListOf()
                list.addAll(user.estates?.map { it as EstateEntity } ?: arrayListOf())
                list
            }
            .flatMapIterable { it }
            .flatMap {
                planContract.getPlanTypes(it.id)
                    .map { plantypes -> plantypes.flatMap { plan -> plan.featuresVariants } }
            }

    fun fetchNotDownloadedPlansSizes(): Observable<Int> =
        fetchAllPlanTypes
            .map { list -> list.sumBy { if (planContract.getPlanFile(it).exists()) 0 else 1 } }
            .scan { t1: Int, t2: Int -> t1 + t2 }

    fun downloadAllPlanTypes(cancellationToken: DownloadUseCase.CancellationToken) =
        fetchAllPlanTypes
            .filter { !cancellationToken.isCancelled }
            .flatMap {downloadPlan(it,cancellationToken)}


    fun downloadPlan(
        list: List<PlanFeaturesVariant>
        , cancellationToken: DownloadUseCase.CancellationToken
    )
            : Observable<DownloadProgress> {
        var runThreads = list.size
        var completedThreads = 0
        return Observable.create<DownloadProgress> { emitter ->
            list.forEach {
                planContract.getPlan(it)
                    .filter { !cancellationToken.isCancelled }
                    .subscribeOn(Schedulers.io())
                    .map { 1 }
                    .scan { t1: Int, t2: Int -> t1.plus(t2) }
                    .subscribe {
                        emitter.onNext(DownloadProgress(it, list.size))
                        completedThreads++
                    }
            }
            while (completedThreads<runThreads && cancellationToken.isCancelled.not())
                Thread.sleep(100)
            if (cancellationToken.isCancelled)
                emitter.onError(Throwable("Cacnelled by user"))
            emitter.onComplete()
        }.subscribeOn(Schedulers.computation())

    }
}




data class DownloadProgress(
    val currentProgress:Int,
    val totalProgress:Int
)